package org.mmc.model.persistence.dao;

import org.mmc.model.domain.AnalysisRequest;
import org.mmc.model.filters.RequestsFilter;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;

import java.util.List;

/**
 * Created on: 2007-10-7
 *
 * @author Stoyo Venkov
 */
public class AnalysisRequestsDaoHbrImpl extends HibernateDaoSupport implements AnalysisRequestsDao {
    public AnalysisRequest getRequestById(Long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(AnalysisRequest.class);
        crit.add(Expression.eq("id", id));
        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (AnalysisRequest) result.get(0);
            }
        }

        return null;
    }

    public List getRequestsByFilter(RequestsFilter filter) {
        if(filter == null) {
            return null;
        }

        DetachedCriteria crit = DetachedCriteria.forClass(AnalysisRequest.class);
        if(filter.getId() != null) {
            crit.add(Expression.eq("id", filter.getUserId()));
        }
        if(filter.getRegFromDate() != null) {
            crit.add(Expression.ge("registrationDate", filter.getRegFromDate()));
        }
        if(filter.getRegToDate() != null) {
            crit.add(Expression.le("registrationDate", filter.getRegToDate()));
        }
        if(filter.getPatientId() != null) {
            crit.add(Expression.eq("patient.id", filter.getPatientId()));
        }
        if(filter.getUserId() != null) {
            crit.add(Expression.eq("creator.id", filter.getUserId()));
        }

        return getHibernateTemplate().findByCriteria(
                crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
    }
}
