package org.mmc.model.persistence.dao;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.mmc.model.domain.Criterion;
import org.mmc.model.filters.CriteriaFilter;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-20
 *
 * @author Stoyo Venkov
 */
public class CriteriaDaoHbrImpl extends HibernateDaoSupport implements CriteriaDao {

    public List getAllCriteria() {
        return getHibernateTemplate().find("from Criterion");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(Criterion.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getCriteriaByFilter(CriteriaFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(Criterion.class);
            if(filter.getName() != null) {
                crit.add(Expression.ilike("name", filter.getName()));
            }
            if(filter.getCode() != null) {
                crit.add(Expression.ilike("code", filter.getCode()));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            crit.addOrder(Order.asc("name"));

            return getHibernateTemplate().findByCriteria(
                    crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
        } else {
            return getAllCriteria();
        }
    }

    public Criterion getCriterionById(long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(Criterion.class);
        crit.add(Expression.eq("id", new Long(id)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Criterion) result.get(0);
            }
        }

        return null;
    }

    public Criterion updateCriterion(Criterion criterion) {
        getHibernateTemplate().update(criterion);
        return getCriterionById(criterion.getId().longValue());
    }

    public Criterion saveCriterion(Criterion criterion) {
        Serializable iId = getHibernateTemplate().save(criterion);
        return getCriterionById(((Long) iId).longValue());
    }
}
