package org.mmc.model.persistence.dao;

import org.mmc.model.domain.Disease;
import org.mmc.model.filters.DiseasesFilter;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;

import java.util.List;
import java.io.Serializable;

/**
 * Hibernate implmentation of DiseasesDao interface.
 *
 * Created on: 2007-9-8
 *
 * @author Stoyo Venkov
 */
public class DiseasesDaoHbrImpl extends HibernateDaoSupport implements DiseasesDao {
    public List getAllDiseases() {
        return getHibernateTemplate().find("from Disease");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(Disease.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getDiseasesByFilter(DiseasesFilter disease) {
        if(disease != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(Disease.class);
            if(disease.getName() != null) {
                crit.add(Expression.ilike("name", disease.getName()));
            }
            if(disease.getCode() != null) {
                crit.add(Expression.ilike("code", disease.getCode()));
            }
            if(disease.getActive() != null) {
                crit.add(Expression.eq("active", disease.getActive()));
            }
            crit.addOrder(Order.asc("name"));

            return getHibernateTemplate().findByCriteria(
                    crit, disease.getCriteriaFirstResult(), disease.getCriteriaMaxResult());
        } else {
            return getAllDiseases();
        }
    }

    public Disease getDiseaseById(long diseaseId) {
        DetachedCriteria crit = DetachedCriteria.forClass(Disease.class);
        crit.add(Expression.eq("id", new Long(diseaseId)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Disease) result.get(0);
            }
        }

        return null;
    }

    public Disease updateDisease(Disease disease) {
        getHibernateTemplate().update(disease);
        return getDiseaseById(disease.getId().longValue());        
    }

    public Disease saveDisease(Disease disease) {
        Serializable iId = getHibernateTemplate().save(disease);
        return getDiseaseById(((Long) iId).longValue());
    }
}
