package org.mmc.model.persistence.dao;

import org.mmc.model.domain.EthnicGroup;
import org.mmc.model.filters.EthnicGroupFilter;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-9
 *
 * @author Stoyo Venkov
 */
public class EthnicGroupDaoHbrImpl extends HibernateDaoSupport implements EthnicGroupsDao {
    public List getAllEthnicGroups() {
        return getHibernateTemplate().find("from EthnicGroup");
    }

    public List getEthnicGroupsByFilter(EthnicGroupFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(EthnicGroup.class);
            if(filter.getName() != null) {
                crit.add(Expression.ilike("name", filter.getName()));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            return getHibernateTemplate().findByCriteria(
                    crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
        } else {
            return getAllEthnicGroups();
        }
    }

    public EthnicGroup getEthnicGroupById(long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(EthnicGroup.class);
        crit.add(Expression.eq("id", new Long(id)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (EthnicGroup) result.get(0);
            }
        }

        return null;
    }

    public EthnicGroup updateEthnicGroup(EthnicGroup group) {
        getHibernateTemplate().update(group);
        return getEthnicGroupById(group.getId().longValue());
    }

    public EthnicGroup saveEthnicGroup(EthnicGroup group) {
        Serializable iId = getHibernateTemplate().save(group);
        return getEthnicGroupById(((Long) iId).longValue());
    }
}
