package org.mmc.model.persistence.dao;

import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.mmc.model.domain.MMCPosition;
import org.mmc.model.filters.PositionFilter;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-9
 *
 * @author Stoyo Venkov
 */
public class MMCPositionsDaoHbrImpl extends HibernateDaoSupport implements MMCPositionsDao {
    public List getAllMMCPositions() {
        return getHibernateTemplate().find("from MMCPosition");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(MMCPosition.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getPositionsByFilter(PositionFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(MMCPosition.class);
            if(filter.getName() != null) {
                crit.add(Expression.ilike("name", filter.getName()));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            return getHibernateTemplate().findByCriteria(
                    crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
        } else {
            return getAllMMCPositions();
        }
    }

    public MMCPosition getPositionById(long positionId) {
        DetachedCriteria crit = DetachedCriteria.forClass(MMCPosition.class);
        crit.add(Expression.eq("id", new Long(positionId)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (MMCPosition) result.get(0);
            }
        }

        return null;
    }

    public MMCPosition updatePosition(MMCPosition position) {
        getHibernateTemplate().update(position);
        return getPositionById(position.getId().longValue());
    }

    public MMCPosition savePosition(MMCPosition position) {
        Serializable iId = getHibernateTemplate().save(position);
//        position.setId((Long) iId);
//        return position;
        return getPositionById(((Long) iId).longValue());
    }
}
