package org.mmc.model.persistence.dao;

import org.mmc.model.domain.Patient;
import org.mmc.model.domain.DossierRecord;
import org.mmc.model.filters.PatientsFilter;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Subqueries;
import org.hibernate.criterion.Property;

import java.util.List;

/**
 * Created on: 2007-10-7
 *
 * @author Stoyo Venkov
 */
public class PatientsDaoHbrImpl extends HibernateDaoSupport implements PatientsDao {
    public Patient getPatientById(Long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(Patient.class);
        crit.add(Expression.eq("id", id));
        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Patient) result.get(0);
            }
        }

        return null;
    }

    public List getPatientsByFilter(PatientsFilter filter) {
        if(filter == null) {
            return null;
        }

        DetachedCriteria crit = DetachedCriteria.forClass(Patient.class);
        if(filter.getFirstName() != null) {
            crit.add(Expression.like("firstName", filter.getFirstName() + "%"));
        }
        if(filter.getFamilyName() != null) {
            crit.add(Expression.like("familyName", filter.getFamilyName() + "%"));
        }
        if(filter.getUniqueNumber() != null) {
            crit.add(Expression.like("uniqueNumber", filter.getUniqueNumber() + "%"));
        }
        if(filter.getInitials() != null) {
            crit.add(Expression.like("initials", filter.getInitials() + "%"));
        }
        if(filter.getSexId() != null) {
            crit.add(Expression.eq("sex.id", filter.getSexId()));
        }
        if(filter.getEthnicGroupId() != null) {
            crit.add(Expression.eq("ethnicGroup.id", filter.getEthnicGroupId()));
        }
        if(filter.getBirthDay() != null) {
            crit.add(Expression.eq("birthDay", filter.getBirthDay()));
        }
        if(filter.getBirthMonth() != null) {
            crit.add(Expression.eq("birthMonth", filter.getBirthMonth()));
        }
        if(filter.getBirthYear() != null) {
            crit.add(Expression.eq("birthYear", filter.getBirthYear()));
        }
        if(filter.getRegFromDate() != null) {
            crit.add(Expression.ge("registrationDate", filter.getRegFromDate()));
        }
        if(filter.getRegToDate() != null) {
            crit.add(Expression.le("registrationDate", filter.getRegToDate()));
        }

        if(filter.getProjectId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(DossierRecord.class);
            projectCrit.add(Expression.eq("project.id", filter.getProjectId()));
            projectCrit.setProjection(Property.forName("id"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }

        return getHibernateTemplate().findByCriteria(
                crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
    }
}
