package org.mmc.model.persistence.dao;

import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.mmc.model.domain.ProbeType;
import org.mmc.model.filters.ProbeTypesFilter;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-9
 *
 * @author Stoyo Venkov
 */
public class ProbeTypesDaoHbrImpl extends HibernateDaoSupport implements ProbeTypesDao {
    public List getAllProbeTypes() {
        return getHibernateTemplate().find("from ProbeType");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(ProbeType.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getProbeTypesByFilter(ProbeTypesFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(ProbeType.class);
            if(filter.getName() != null) {
                crit.add(Expression.ilike("name", filter.getName()));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            return getHibernateTemplate().findByCriteria(crit);
        } else {
            return getAllProbeTypes();
        }
    }

    public ProbeType getProbeTypeById(long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(ProbeType.class);
        crit.add(Expression.eq("id", new Long(id)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (ProbeType) result.get(0);
            }
        }

        return null;
    }

    public ProbeType updateProbeType(ProbeType type) {
        getHibernateTemplate().update(type);
        return getProbeTypeById(type.getId().longValue());
    }

    public ProbeType saveProbeType(ProbeType type) {
        Serializable iId = getHibernateTemplate().save(type);
        type.setId((Long) iId);
        return type;
    }
}
