package org.mmc.model.persistence.dao;

import org.mmc.model.domain.*;
import org.mmc.model.filters.ProbesFilter;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Subqueries;

import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;

/**
 * Created on: 2007-10-9
 *
 * @author Stoyo Venkov
 */
public class ProbesDaoHbrImpl extends HibernateDaoSupport implements ProbesDao {
    private ConfParamsDao paramsDao;

    public ConfParamsDao getParamsDao() {
        return paramsDao;
    }

    public void setParamsDao(ConfParamsDao paramsDao) {
        this.paramsDao = paramsDao;
    }

    public Probe getProbeById(Long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(Probe.class);
        crit.add(Expression.eq("id", id));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Probe) result.get(0);
            }
        }

        return null;
    }

    public List getProbesByFilter(ProbesFilter filter) {
        if(filter == null) {
            return null;
        }

        DetachedCriteria crit = createCriteriaByFilter(filter);

        return getHibernateTemplate().findByCriteria(
                crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
    }

    public Probe saveProbe(Probe probe) {
        Serializable iId = getHibernateTemplate().save(probe);
        return getProbeById((Long) iId);
    }

    public Probe updateProbe(Probe probe) {
        getHibernateTemplate().update(probe);
        return getProbeById(probe.getId());
    }

    public Location getLocationById(Long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(Location.class);
        crit.add(Expression.eq("id", id));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Location) result.get(0);
            }
        }

        return null;
    }

    public Location getFirstLocation(Long stepId) {
        DetachedCriteria crit = DetachedCriteria.forClass(Location.class);
        crit.add(Expression.eq("stepId", stepId));
        List locations = getHibernateTemplate().findByCriteria(crit);
        if(locations != null && locations.size() > 0) {
            return (Location) locations.get(0);
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public List getChildrenLocations(Long locationId, Long projectId) {
        DetachedCriteria movementCrit = DetachedCriteria.forClass(LocationChange.class);
        movementCrit.add(Expression.eq("srcId", locationId));
        movementCrit.setProjection(Property.forName("destId"));

        DetachedCriteria crit = DetachedCriteria.forClass(Location.class);
        crit.add(Subqueries.propertyIn("id", movementCrit));

        DetachedCriteria stepCriteria = DetachedCriteria.forClass(Step.class);
        if(projectId != null) {
            stepCriteria.add(Expression.eq("projectId", projectId));
        } else {
            stepCriteria.add(Expression.isNull("projectId"));
        }
        stepCriteria.setProjection(Property.forName("id"));
        crit.add(Subqueries.propertyIn("stepId", stepCriteria));

        return getHibernateTemplate().findByCriteria(crit);
    }

    public Step getStepById(Long id) {
        DetachedCriteria crit = DetachedCriteria.forClass(Step.class);
        crit.add(Expression.eq("id", id));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Step) result.get(0);
            }
        }

        return null;
    }

    public Step saveStep(Step step) {
        Serializable id = getHibernateTemplate().save(step);
        step.setId((Long) id);
        return step;
    }

    public Step updateStep(Step step) {
        getHibernateTemplate().update(step);
        return step;
    }

    public Location saveLocation(Location location) {
        Serializable id = getHibernateTemplate().save(location);
        location.setId((Long) id);
        return location;
    }

    public void registerMovement(LocationChange lc) {
        getHibernateTemplate().save(lc);
    }

    public List getProbeSteps(Probe probe) {
        List result = new ArrayList();

        String hql = "from Step where probe_id = ? and project_id is null order by startDate asc";
        List steps = getHibernateTemplate().find(hql, probe.getId());
        StepsInProject sp = new StepsInProject();
        sp.setSteps(steps);
        result.add(sp);

        for (int i = 0; i < probe.getProbeProjects().size(); i++) {
            ProbeProject probeProject = (ProbeProject) probe.getProbeProjects().get(i);
            hql = "from Step where probe_id = ? and project_id = ? order by startDate asc";
            steps = getHibernateTemplate().find(hql, new Object[] {probeProject.getProbeId(), probeProject.getProject().getId()});
            if(steps != null && steps.size() > 0) {
                sp = new StepsInProject();
                sp.setProjectId(probeProject.getProject().getId());
                sp.setProjectName(probeProject.getProject().getName());
                sp.setSteps(steps);
                result.add(sp);
            }
        }

        return result;
    }

    public List getAllProbesByFilter(ProbesFilter filter) {
        if(filter == null) {
            return null;
        }

        DetachedCriteria crit = createCriteriaByFilter(filter);

        return getHibernateTemplate().findByCriteria(crit);
    }

    private DetachedCriteria createCriteriaByFilter(ProbesFilter filter) {
        DetachedCriteria crit = DetachedCriteria.forClass(Probe.class);
        if(filter.getId() != null) {
            crit.add(Expression.eq("id", filter.getId()));
        }
        if(filter.getRegFromDate() != null) {
            crit.add(Expression.ge("registrationDate", filter.getRegFromDate()));
        }
        if(filter.getRegToDate() != null) {
            crit.add(Expression.le("registrationDate", filter.getRegToDate()));
        }
        if(filter.getCreatorId() != null) {
            crit.add(Expression.eq("creator.id", filter.getCreatorId()));
        }
        if(filter.getSenderId() != null) {
            DetachedCriteria requestCrit = DetachedCriteria.forClass(AnalysisRequest.class);
            requestCrit.add(Expression.eq("sender.id", filter.getSenderId()));
            requestCrit.setProjection(Property.forName("id"));
            crit.add(Subqueries.propertyIn("request.id", requestCrit));
        }
        if(filter.getPatientId() != null) {
            crit.add(Expression.eq("patient.id", filter.getPatientId()));
        }
        if(filter.getRequestId() != null) {
            crit.add(Expression.eq("request.id", filter.getRequestId()));
        }
        if(filter.getProjectId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(ProbeProject.class);
            projectCrit.add(Expression.eq("project.id", filter.getProjectId()));
            projectCrit.setProjection(Property.forName("probeId"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }
        if(filter.getTubeId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(Location.class);
            projectCrit.add(Expression.eq("tube.id", filter.getTubeId()));
            projectCrit.setProjection(Property.forName("probeId"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }
        if(filter.getRackId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(Location.class);
            projectCrit.add(Expression.eq("rack.id", filter.getRackId()));
            projectCrit.setProjection(Property.forName("probeId"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }
        if(filter.getPlateId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(Location.class);
            projectCrit.add(Expression.eq("plate.id", filter.getPlateId()));
            projectCrit.setProjection(Property.forName("probeId"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }
        if(filter.getBatchGroupId() != null) {
            DetachedCriteria projectCrit = DetachedCriteria.forClass(Location.class);
            projectCrit.add(Expression.eq("group.id", filter.getBatchGroupId()));
            projectCrit.setProjection(Property.forName("probeId"));
            crit.add(Subqueries.propertyIn("id", projectCrit));
        }

        if(filter.getTerm() != null) {
            if(ProbesFilter.EXPIRED_PROBES.equals(filter.getTerm())) {
                crit.add(Expression.eq("expired", Boolean.TRUE));
                crit.add(Expression.isNull("finishDate"));
            } else if(ProbesFilter.PRESSING_PROBES.equals(filter.getTerm())) {
                String alarmDaysStr = paramsDao.getParamValue(ConfParam.ALARM_TERM_PARAM);
                Integer alarmDays = Integer.valueOf(alarmDaysStr);
                crit.add(Expression.le("daysToCompletion", alarmDays));
                crit.add(Expression.ge("daysToCompletion", new Integer(0)));
                crit.add(Expression.isNull("finishDate"));
            }
        }

        return crit;
    }
}
