package org.mmc.model.persistence.dao;

import org.mmc.model.filters.ProjectsFilter;
import org.mmc.model.domain.Project;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-19
 *
 * @author Stoyo Venkov
 */
public class ProjectsDaoHbrImpl extends HibernateDaoSupport implements ProjectsDao {
    public List getAllProjects() {
        return getHibernateTemplate().find("from Project");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(Project.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getProjectsByFilter(ProjectsFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(Project.class);
            if(filter.getName() != null) {
                crit.add(Expression.ilike("name", filter.getName()));
            }
            if(filter.getCode() != null) {
                crit.add(Expression.ilike("code", filter.getCode()));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            crit.addOrder(Order.asc("name"));

            return getHibernateTemplate().findByCriteria(
                    crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
        } else {
            return getAllProjects();
        }
    }

    public Project getProjectById(long projectId) {
        DetachedCriteria crit = DetachedCriteria.forClass(Project.class);
        crit.add(Expression.eq("id", new Long(projectId)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (Project) result.get(0);
            }
        }

        return null;
    }

    public Project updateProject(Project project) {
        getHibernateTemplate().update(project);
        return getProjectById(project.getId().longValue());
    }

    public Project saveProject(Project project) {
        Serializable iId = getHibernateTemplate().save(project);
        return getProjectById(((Long) iId).longValue());
    }
}
