package org.mmc.model.persistence.dao;

import org.mmc.model.filters.UserFilter;
import org.mmc.model.domain.User;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;

import java.util.List;
import java.io.Serializable;

/**
 * Created on: 2007-9-18
 *
 * @author Stoyo Venkov
 */
public class UserDaoHbrImpl extends HibernateDaoSupport implements UserDao {
    public List getAllUsers() {
        return getHibernateTemplate().find("from User");
    }

    public List getAllActive() {
        DetachedCriteria crit = DetachedCriteria.forClass(User.class);
        crit.add(Expression.eq("active", Boolean.TRUE));
        return getHibernateTemplate().findByCriteria(crit);
    }

    public List getUserByFilter(UserFilter filter) {
        if(filter != null) {
            DetachedCriteria crit = DetachedCriteria.forClass(User.class);
            if(filter.getFirstName() != null) {
                crit.add(Expression.like("firstName", filter.getFirstName() + "%"));
            }
            if(filter.getFamilyName() != null) {
                crit.add(Expression.like("familyName", filter.getFamilyName() + "%"));
            }
            if(filter.getUserName() != null) {
                crit.add(Expression.like("userName", filter.getUserName() + "%"));
            }
            if(filter.getActive() != null) {
                crit.add(Expression.eq("active", filter.getActive()));
            }
            return getHibernateTemplate().findByCriteria(
                    crit, filter.getCriteriaFirstResult(), filter.getCriteriaMaxResult());
        } else {
            return getAllUsers();
        }
    }

    public User getUserById(long userId) {
        DetachedCriteria crit = DetachedCriteria.forClass(User.class);
        crit.add(Expression.eq("id", new Long(userId)));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (User) result.get(0);
            }
        }

        return null;
    }

    public User getUserByUsernameAndPass(String password, String username) {
        DetachedCriteria crit = DetachedCriteria.forClass(User.class);
        crit.add(Expression.eq("userName", username));
        crit.add(Expression.eq("password", password));

        List result = getHibernateTemplate().findByCriteria(crit);

        if(result != null) {
            if(result.size() > 0) {
                return (User) result.get(0);
            }
        }

        return null;
    }

    public User updateUser(User user) {
        getHibernateTemplate().update(user);
        return getUserById(user.getId().longValue());
    }

    public User saveUser(User user) {
        Serializable iId = getHibernateTemplate().save(user);
        return getUserById(((Long) iId).longValue());
    }

    public List getAllRoles() {
        return getHibernateTemplate().find("from Role");
    }
}
