package org.mmc.model.print;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.mmc.model.domain.Probe;
import org.mmc.utils.ParseUtils;

import java.util.List;
import java.lang.reflect.InvocationTargetException;
import java.io.*;

/**
 * Created on: 2007-10-18
 *
 * @author Stoyo Venkov
 */
public class ProbesToXMLConvertor {
    private List probes;
    private List properties;
    private List params;
    private String name;
    private String creatorName;

    public ProbesToXMLConvertor(String name, List params, List properties, List probes, String creatorName) {
        this.params = params;
        this.probes = probes;
        this.name = name;
        this.properties = properties;
        this.creatorName = creatorName;
    }

    public Document createTransformableDocument() {
        Document doc = new DocumentImpl();

        Element root = doc.createElement("artifact");

        root.appendChild(createHeader(doc));

        for (int i = 0; i < params.size(); i++) {
            NameValuePair nameValuePair = (NameValuePair) params.get(i);
            root.appendChild(createParagraph(doc, nameValuePair));
        }

        root.appendChild(createTable(doc));

        doc.appendChild(root);
        return doc;
    }

    public InputStream createXML() {
        Document doc = createTransformableDocument();

        OutputFormat format = new OutputFormat(doc, "UTF-8", true);

        try{
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            XMLSerializer serialFile = new XMLSerializer(out, format);
            serialFile.asDOMSerializer();
            serialFile.serialize(doc);
            return new ByteArrayInputStream(out.toByteArray());
        } catch (IOException e){
            e.printStackTrace();
        }
        return null;
    }

    private Element createHeader(Document doc) {
        Element header = doc.createElement("header");
        Element p = doc.createElement("p");
        header.appendChild(p);
        p.appendChild(doc.createTextNode(name));
        return header;
    }

    private Element createParagraph(Document doc, NameValuePair pair) {
        Element p = doc.createElement("p");
        p.appendChild(doc.createTextNode(pair.getName() + ": " + pair.getValue()));
        return p;
    }

    private Element createTable(Document doc) {
        Element table = doc.createElement("table");

        Element thead = doc.createElement("thead");
        table.appendChild(thead);
        Element theadTr = doc.createElement("tr");
        thead.appendChild(theadTr);
        for (int i = 0; i < properties.size(); i++) {
            ColumnProps columnProps = (ColumnProps) properties.get(i);
            Element td = doc.createElement("td");
            td.setAttribute("width", Long.toString(columnProps.getWidth()) + "%");
            td.appendChild(doc.createTextNode(columnProps.getColumnName()));
            theadTr.appendChild(td);
        }

        Element tbody = doc.createElement("tbody");
        table.appendChild(tbody);

        for (int i = 0; i < probes.size(); i++) {
            Probe probe = (Probe) probes.get(i);
            Element tr = doc.createElement("tr");
            tbody.appendChild(tr);
            for (int j = 0; j < properties.size(); j++) {
                ColumnProps columnProps = (ColumnProps) properties.get(j);
                Element td = createCell(doc, probe, columnProps);
                tr.appendChild(td);
            }
        }

        return table;
    }

    private Element createCell(Document doc, Probe probe, ColumnProps columnProps) {
        Element td = doc.createElement("td");

        String cellValue = "";
        try {
            Class typeClazz = PropertyUtils.getPropertyType(probe, columnProps.getPropertyName());
            if(Boolean.class.getName().equals(typeClazz.getName())) {
                Boolean value = (Boolean) PropertyUtils.getProperty(probe, columnProps.getPropertyName());
                cellValue = ParseUtils.parseToBgYN(value);
            } else {
                cellValue = BeanUtils.getProperty(probe, columnProps.getPropertyName());
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (InvocationTargetException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (NoSuchMethodException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        td.appendChild(doc.createTextNode(cellValue));

        return td;
    }
}
