package org.mmc.model.service;

import org.mmc.model.filters.CriteriaFilter;
import org.mmc.model.domain.Criterion;

import java.util.List;

/**
 * Created on: 2007-9-20
 *
 * @author Stoyo Venkov
 */
public interface CriteriaService {
    /**
     * Gets all criteria ordered by name ascending.
     *
     * @return List of all ethnic groups.
     */
    public List getAllCriteria();

    /**
     * Gets all criteria that match the filter ordered by name ascending.
     *
     * @param filter Filter object.
     * @return List of filtered criteria.
     */
    public List getCriteriaByFilter(final CriteriaFilter filter);

    /**
     * Gets a criterion by its id.
     *
     * @param id Criterion id.
     * @return The criterion.
     */
    public Criterion getCriterionById(long id);

    public Criterion updateCriterion(Criterion criterion);
    public Criterion saveCriterion(Criterion criterion);

    /**
     * Saves or updates the criterion. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param criterion The criterion to be saved.
     * @return The same criterion with supplied Id if the criterion was saved.
     */
    public Criterion saveOrUpdateCriterion(Criterion criterion);
}
