package org.mmc.model.service;

import org.mmc.model.domain.Disease;
import org.mmc.model.filters.DiseasesFilter;

import java.util.List;

/**
 * Created on: 2007-9-8
 *
 * @author Stoyo Venkov
 */
public interface DiseasesService {
    /**
     * Gets all diseases ordered by name ascending.
     *
     * @return List of all diseases.
     */
    public List getAllDiseases();

    /**
     * Gets all active diseases.
     *
     * @return List of all active diseases.
     */
    public List getAllActive();
    /**
     * Gets all diseases that match the filter ordered by name ascending.
     *
     * @param disease Filter object.
     * @return List of filtered diseases.
     */
    public List getDiseasesByFilter(final DiseasesFilter disease);

    /**
     * Gets a disease by its id.
     *
     * @param diseaseId Disease id.
     * @return The disease.
     */
    public Disease getDiseaseById(long diseaseId);

    public Disease updateDisease(Disease disease);
    public Disease saveDisease(Disease disease);

    /**
     * Saves or updates the disease. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param disease The disease to be saved.
     * @return The same disease with supplied Id if the disease was saved.
     */
    public Disease saveOrUpdateDisease(Disease disease);
}
