package org.mmc.model.service;

import org.mmc.model.filters.EthnicGroupFilter;
import org.mmc.model.domain.EthnicGroup;

import java.util.List;

/**
 * Created on: 2007-9-16
 *
 * @author Stoyo Venkov
 */
public interface EthnicGroupsService {
    /**
     * Gets all ethnic groups ordered by name ascending.
     *
     * @return List of all ethnic groups.
     */
    public List getAllEthnicGroups();

    /**
     * Gets all ethnic groups that match the filter ordered by name ascending.
     *
     * @param filter Filter object.
     * @return List of filtered ethnic groups.
     */
    public List getEthnicGroupsByFilter(final EthnicGroupFilter filter);

    /**
     * Gets a ethnic group by its id.
     *
     * @param id Ethnic group id.
     * @return The ethnic group.
     */
    public EthnicGroup getEthnicGroupById(long id);

    public EthnicGroup updateEthnicGroup(EthnicGroup group);
    public EthnicGroup saveEthnicGroup(EthnicGroup group);

    /**
     * Saves or updates the group. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param group The group to be saved.
     * @return The same group with supplied Id if the group was saved.
     */
    public EthnicGroup saveOrUpdateEthnicGroup(EthnicGroup group);
}
