package org.mmc.model.service;

import org.mmc.model.domain.MMCPosition;
import org.mmc.model.filters.PositionFilter;

import java.util.List;

/**
 * Created on: 2007-9-9
 *
 * @author Stoyo Venkov
 */
public interface MMCPositionsService {
    /**
     * Gets all positions ordered by name ascending.
     *
     * @return List of all positions.
     */
    public List getAllPositions();

    /**
     * Gets all active positions ordered by name ascending.
     *
     * @return List of all active positions.
     */
    public List getAllActive();
    /**
     * Gets all positions that match the filter ordered by name ascending.
     *
     * @param filter Filter object.
     * @return List of filtered positions.
     */
    public List getPositionsByFilter(final PositionFilter filter);

    /**
     * Gets a position by its id.
     *
     * @param positionId Position id.
     * @return The position.
     */
    public MMCPosition getPositionById(long positionId);

    public MMCPosition updatePosition(MMCPosition position);
    public MMCPosition savePosition(MMCPosition position);

    /**
     * Saves or updates the position. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param position The position to be saved.
     * @return The same position with supplied Id if the position was saved.
     */
    public MMCPosition saveOrUpdatePosition(MMCPosition position);
}
