package org.mmc.model.service;

import org.mmc.model.filters.ProbeTypesFilter;
import org.mmc.model.domain.ProbeType;

import java.util.List;

/**
 * Created on: 2007-9-20
 *
 * @author Stoyo Venkov
 */
public interface ProbeTypesService {
    /**
     * Gets all probeTypes ordered by name ascending.
     *
     * @return List of all probeTypes.
     */
    public List getAllProbeTypes();

    /**
     * Gets all active probeTypes.
     *
     * @return List of all active probeTypes.
     */
    public List getAllActive();
    /**
     * Gets all probeTypes that match the filter ordered by name ascending.
     *
     * @param probeType Filter object.
     * @return List of filtered probeTypes.
     */
    public List getProbeTypesByFilter(final ProbeTypesFilter probeType);

    /**
     * Gets a probeType by its id.
     *
     * @param probeTypeId ProbeType id.
     * @return The probeType.
     */
    public ProbeType getProbeTypeById(long probeTypeId);

    public ProbeType updateProbeType(ProbeType probeType);
    public ProbeType saveProbeType(ProbeType probeType);

    /**
     * Saves or updates the probeType. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param probeType The probeType to be saved.
     * @return The same probeType with supplied Id if the probeType was saved.
     */
    public ProbeType saveOrUpdateProbeType(ProbeType probeType);
}
