package org.mmc.model.service;

import org.mmc.model.domain.Probe;
import org.mmc.model.domain.User;
import org.mmc.model.domain.Step;
import org.mmc.model.domain.Location;
import org.mmc.model.filters.ProbesFilter;

import java.util.List;
import java.util.Date;

/**
 * Created on: 2007-10-9
 *
 * @author Stoyo Venkov
 */
public interface ProbesService {
    public NomenclaturesService getNomenclaturesService();
    public void setNomenclaturesService(NomenclaturesService nomenclaturesService);

    /**
     * Gets a probe by its id.
     *
     * @param id The id to be searched for.
     * @return The probe.
     */
    public Probe getProbeById(Long id);

    /**
     * Gets all probes that match the filter.
     *
     * @param filter Filter object.
     * @return List of filtered probes.
     */
    public List getProbesByFilter(ProbesFilter filter);

    public Probe updateProbe(Probe probe);
    public Probe registerProbe(Probe probe);
    public Probe registerProbe(Long userId, Long patientId, Long requestId, List criteria,
                               List projects, Integer termInDays, Long probeTypeId, Long tubeId);

    /**
     * Registers new process step. Marks the previous step as completed, if the previous step is supplied and has not been yet completed.
     * @param userId
     * @param prevStepId
     * @param probeId
     * @param projectId
     * @param kindId
     * @param notes
     * @return
     */
    public Step registerStep(Long userId, Long prevStepId, Long probeId, Long projectId, Long kindId, String notes);
    /**
     * Registers location. If previous location id is provieded, then location movement is also registered.
     * @param probeId
     * @param stepId
     * @param prevLocationId
     * @param tubeId
     * @param rackId
     * @param plateId
     * @param batchGroupId
     * @param coordinates
     * @return
     */
    public Location registerLocation(Long probeId, Long stepId, Long prevLocationId,
                                     Long tubeId, Long rackId, Long plateId, Long batchGroupId, String coordinates);
    public String getLocationsTable(Long stepId);

    public Probe cancelProbe(Probe probe, String reason);
    public Probe markProbeAsSent(Probe probe);
    public Probe completeProbe(Probe probe);
    public byte[] executeProjectReport(Long projectId, Date regFromDate, Date regToDate);
    public byte[] executePatientReport(Long patientId, Date regToDate);
    public byte[] executeExpiredReport(Long creatorId, Long projectId);
    public byte[] executePressingReport(Long creatorId, Long projectId);
}
