package org.mmc.model.service;

import org.mmc.model.domain.Project;
import org.mmc.model.filters.ProjectsFilter;

import java.util.List;

/**
 * Created on: 2007-9-19
 *
 * @author Stoyo Venkov
 */
public interface ProjectsService {
    /**
     * Gets all projects ordered by name ascending.
     *
     * @return List of all projects.
     */
    public List getAllProjects();

    /**
     * Gets all active projects ordered by name ascending.
     *
     * @return List of all active projects.
     */
    public List getAllActive();
    /**
     * Gets all projects that match the filter ordered by name ascending.
     *
     * @param filter Filter object.
     * @return List of filtered projects.
     */
    public List getProjectsByFilter(final ProjectsFilter filter);

    /**
     * Gets a project by its id.
     *
     * @param projectId Project id.
     * @return The project.
     */
    public Project getProjectById(long projectId);

    public Project updateProject(Project project);
    public Project saveProject(Project project);

    /**
     * Saves or updates the project. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param project The project to be saved.
     * @return The same project with supplied Id if the project was saved.
     */
    public Project saveOrUpdateProject(Project project);
}
