package org.mmc.model.service;

import org.mmc.model.filters.UserFilter;
import org.mmc.model.domain.User;

import java.util.List;

/**
 * Created on: 2007-9-18
 *
 * @author Stoyo Venkov
 */
public interface UsersService {
    /**
     * Gets the mmc positions service.
     * @return The positions service.
     */
    public MMCPositionsService getPositionsService();
    /**
     * Gets all users that match the filter ordered by name ascending.
     *
     * @param filter Filter object.
     * @return List of filtered users.
     */
    public List getUsersByFilter(final UserFilter filter);

    /**
     * Gets a user by its id.
     *
     * @param userId Position id.
     * @return The user.
     */
    public User getUserById(long userId);

    /**
     * Gets an active user by its username and password.
     *
     * @param password User's pass.
     * @param username User's username.
     * @return The user if found, null - otherwise.
     */
    public User getUserByUsernameAndPass(final String password, final String username);

    public User updateUser(User user);
    public User saveUser(User user);

    /**
     * Saves or updates the user. Decides to save if the <code>id</code> field is not supplied.
     *
     * @param user The user to be saved.
     * @return The same user with supplied Id if the user was saved.
     */
    public User saveOrUpdateUser(User user);

    public List getAllUsers();

    /**
     * Gets all roles.
     * @return The list of all roles.
     */
    public List getAllRoles();
}
