package org.mmc.utils;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

/**
 * Created by: Stoysky
 * Date: 2007-1-4
 * Time: 17:26:12
 *
 * @author <a href="mailto:svenkov@vereo.bg">Stoyo Venkov</a>
 */
public class DateTimeUtils {
    public static final String LAST_TIME_IN_DAY = "23:59:59";
    public static final String EMPTY_STRING = " ";
    public static final String DATE_FORMAT = "dd.MM.yyyy";
    public static final String TIME_FORMAT = "HH:mm";
    public static final String TIME_WITH_SECONDS_FORMAT = TIME_FORMAT + ":ss";
    public static final String DATE_TIME_FORMAT = DATE_FORMAT + EMPTY_STRING + TIME_FORMAT;
    public static final String DATE_TIME_SECONDS_FORMAT = DATE_FORMAT + EMPTY_STRING +
                                                          TIME_WITH_SECONDS_FORMAT;

    public static final String YEAR_MONTH_FORMAT = "MM.yyyy";
    public static final String XML_TIME_FORMAT = "HHmm";

    private static final SimpleDateFormat aDateFormater = new SimpleDateFormat(DATE_FORMAT);
    private static final SimpleDateFormat aTimeFormater = new SimpleDateFormat(TIME_FORMAT);
    private static final SimpleDateFormat aDateTimeFormater =
            new SimpleDateFormat(DATE_TIME_FORMAT);
    private static final SimpleDateFormat aDateTimeSecondsFormater =
            new SimpleDateFormat(DATE_TIME_SECONDS_FORMAT);
    private static final SimpleDateFormat aYearMonthFormater =
            new SimpleDateFormat(YEAR_MONTH_FORMAT);

    static {
        aDateFormater.setLenient(false);
        aTimeFormater.setLenient(false);
        aDateTimeFormater.setLenient(false);
        aDateTimeSecondsFormater.setLenient(false);
    }

    /**
     * Formats a given <code>Date</code> to a <code>String</code> in format {@link #DATE_FORMAT}
     *
     * @param date
     *
     * @return String
     */
    public static String formatDate(Date date) {
        return (date != null ? aDateFormater.format(date) : EMPTY_STRING);
    }

    public static String formatDateNoDelim(Date date) {
        return (date != null ? new SimpleDateFormat("yyyyMMdd").format(date) : EMPTY_STRING);
    }

    /**
     * Formats a given <code>Date</code> to a <code>String</code> in format {@link #TIME_FORMAT}
     *
     * @param date
     *
     * @return
     */
    public static String formatTime(Date date) {
        return (date != null ? aTimeFormater.format(date) : EMPTY_STRING);
    }

    /**
     * Formats a given <code>Date</code> to a <code>String</code>
     * in format {@link #DATE_TIME_FORMAT}
     *
     * @param date
     *
     * @return String
     */
    public static String formatDateTime(Date date) {
        return (date != null ? aDateTimeFormater.format(date) : EMPTY_STRING);
    }

    public static String formatYearMonth(Date date) {
        return aYearMonthFormater.format(date);
    }

    public static String formatXMLTime(Date date) {
        return new SimpleDateFormat(XML_TIME_FORMAT).format(date);
    }

    /**
     * Parses a given <code>String</code> in format {@link #DATE_FORMAT} to a <code>Date</code>.
     * The time is set to 00:00:00
     * @param strDate
     * @return
     * @throws ParseException
     */
    public static Date getDate(String strDate) throws ParseException {
        return aDateFormater.parse(strDate);
    }

    /**
     * Parses a given <code>String</code> in format {@link #DATE_FORMAT} to a <code>Date</code>.
     * The time is set to 23:59:59
     * @param strDate
     * @return
     * @throws ParseException
     */
    public static Date getDateEndOfDay(String strDate) throws ParseException {
        return aDateTimeSecondsFormater.parse(strDate + EMPTY_STRING  + LAST_TIME_IN_DAY);
    }

    public static Date getDate(String strDate, String strFormater) throws ParseException {
        return new SimpleDateFormat(strFormater).parse(strDate);
    }
}