package org.mmc.utils;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParsePosition;

/**
 * Created by: Stoysky
 * Date: 2007-1-4
 * Time: 17:26:50
 *
 * @author <a href="mailto:svenkov@vereo.bg">Stoyo Venkov</a>
 */
public class NumberUtils  {

    public NumberUtils() {
    }

    public static String formatAmount(BigDecimal amount) {
        NumberFormat fmt = NumberFormat.getInstance();
        DecimalFormat dfmt = (DecimalFormat)fmt;
        DecimalFormatSymbols ds = new DecimalFormatSymbols();
        ds.setDecimalSeparator(',');
        ds.setGroupingSeparator(' ');
        dfmt.setDecimalFormatSymbols(ds);
        dfmt.applyPattern("#,##0.00");
        dfmt.setMaximumFractionDigits(2);
        return dfmt.format(amount);
    }

    public static String formatAmountNoSpacesBetweenThousands(BigDecimal amount) {

        if (amount == null) {
            return "";
        }

        NumberFormat fmt = NumberFormat.getInstance();
        DecimalFormat dfmt = (DecimalFormat)fmt;
        DecimalFormatSymbols ds = new DecimalFormatSymbols();
        ds.setDecimalSeparator(',');
        dfmt.setDecimalFormatSymbols(ds);
        dfmt.applyPattern("0.00");
        dfmt.setMaximumFractionDigits(2);
        return dfmt.format(amount);
    }

    public static String formatFixingNoSpacesBetweenThousands(BigDecimal amount) {

        if (amount == null) {
            return "";
        }

        NumberFormat fmt = NumberFormat.getInstance();
        DecimalFormat dfmt = (DecimalFormat)fmt;
        DecimalFormatSymbols ds = new DecimalFormatSymbols();
        ds.setDecimalSeparator(',');
        dfmt.setDecimalFormatSymbols(ds);
        dfmt.applyPattern("0.000000");
        dfmt.setMaximumFractionDigits(6);
        return dfmt.format(amount);
    }

    private static DecimalFormat getCurrencyFormatter()
	{
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setGroupingSeparator(' ');
		dfs.setDecimalSeparator(',');
		DecimalFormat df = new DecimalFormat("#,##0.##", dfs);
        df.setMaximumFractionDigits(2);
        df.setMinimumFractionDigits(2);

		return df;
	}

    private static DecimalFormat getBigDecimalFormatterComma()
	{
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator(',');
        dfs.setGroupingSeparator(' ');
		DecimalFormat df = new DecimalFormat("#,##0.##", dfs);
        df.setMaximumFractionDigits(2);
        df.setMinimumFractionDigits(2);

		return df;
	}

    public static String formatCurrency(BigDecimal amount){
        if (amount == null){
            return "";
        }

		return getCurrencyFormatter().format(amount);
	}

    public static BigDecimal parseCurrency( String pValue )
	{
		DecimalFormat f = getCurrencyFormatter();
		Number d = null;
		ParsePosition pos = new ParsePosition(0);
		d = f.parse(pValue,pos);
		if ( (d != null) && (pos.getIndex() == pValue.length()))
        {
            return new BigDecimal(d.doubleValue());
        }
		else
			throw new NumberFormatException("Cannot parse string as bigDecimal");
	}

    public static BigDecimal parseCurrency( String pValue, int intDigits )
	{
		DecimalFormat f = getCurrencyFormatter();
        f.setMaximumIntegerDigits(intDigits);
        f.setMinimumIntegerDigits(intDigits);
		Number d = null;
		ParsePosition pos = new ParsePosition(0);
		d = f.parse(pValue,pos);
		if ( (d != null) && (pos.getIndex() == pValue.length()))
        {
            return new BigDecimal(d.doubleValue());
        }
		else
			throw new NumberFormatException("Cannot parse string as bigDecimal");
	}

    private static DecimalFormat getPercentFormatter()
	{
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator(',');
		DecimalFormat df = (DecimalFormat) NumberFormat.getPercentInstance();
        df.applyPattern("##0.##");
        df.setDecimalFormatSymbols(dfs);
        df.setMaximumFractionDigits(2);
        df.setMinimumFractionDigits(2);

		return df;
	}

    public static String formatPercent(BigDecimal amount){
        if (amount == null){
            return "";
        }

		return getPercentFormatter().format(amount);
	}

    public static BigDecimal parsePercent( String pValue )
	{
		DecimalFormat f = getPercentFormatter();
		Number d = null;
		ParsePosition pos = new ParsePosition(0);
		d = f.parse(pValue,pos);
		if ( (d != null) && (pos.getIndex() == pValue.length()))
			return new BigDecimal(d.doubleValue());
		else
			throw new NumberFormatException("Cannot parse string as bigDecimal");
	}

    private static DecimalFormat getIntegerFormatter()
	{
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setGroupingSeparator(' ');
		DecimalFormat df = (DecimalFormat) NumberFormat.getIntegerInstance();
        df.applyPattern("#,##0");
        df.setDecimalFormatSymbols(dfs);

		return df;
	}

    public static String formatInteger(BigDecimal amount){
        if (amount == null){
            return "";
        }

		return getIntegerFormatter().format(amount);
	}

    public static BigDecimal parseInteger( String pValue )
	{
		DecimalFormat f = getIntegerFormatter();
		Number d = null;
		ParsePosition pos = new ParsePosition(0);
		d = f.parse(pValue,pos);
		if ( (d != null) && (pos.getIndex() == pValue.length()))
			return new BigDecimal(d.intValue());
		else
			throw new NumberFormatException("Cannot parse string as bigDecimal");
	}

    public static BigDecimal parseBigDecimalComma(String pValue) {
		DecimalFormat f = getBigDecimalFormatterComma();
		Number d = null;
		ParsePosition pos = new ParsePosition(0);
		d = f.parse(pValue,pos);
		if ( (d != null) && (pos.getIndex() == pValue.length()))
			return new BigDecimal(d.doubleValue());
		else
			throw new NumberFormatException("Cannot parse string as bigDecimal");
    }

    public static String formatBigDecimalComma(BigDecimal amount) {
		if(amount == null)
            return "";

        return getBigDecimalFormatterComma().format(amount);
    }

    public static Long parseLong(String value) {
        if(value == null) {
            return null;
        }
        return new Long(value);
    }

    public static String getFiveDigitNumber(Integer number) {
        final Integer maxSize = new Integer(99999);
        if (number.compareTo(maxSize) == 1) {
            throw new RuntimeException("Number greater than max allowed sie of " + maxSize);
        }

        final String strNumber = number.toString();
        final int maxStringLength = 5;
        final int gaps = maxStringLength - strNumber.length();
        final String strZero = "0";
        StringBuffer strBuffer = new StringBuffer(maxStringLength);

        if (strNumber.length() < maxStringLength) {
            for (int i = 0; i < gaps; i++) {
                strBuffer.append(strZero);
            }
            strBuffer.append(strNumber);
        }

        return strBuffer.toString();
    }

    public static String getSixDigitNumber(Integer number) {
        final Integer maxSize = new Integer(999999);
        if (number.compareTo(maxSize) == 1) {
            throw new RuntimeException("Number greater than max allowed sie of " + maxSize);
        }

        final String strNumber = number.toString();
        final int maxStringLength = 6;
        final int gaps = maxStringLength - strNumber.length();
        final String strZero = "0";
        StringBuffer strBuffer = new StringBuffer(maxStringLength);

        if (strNumber.length() < maxStringLength) {
            for (int i = 0; i < gaps; i++) {
                strBuffer.append(strZero);
            }
            strBuffer.append(strNumber);
        }

        return strBuffer.toString();
    }

    public static String formatAmountPoint(float amount) {
        DecimalFormat format = new DecimalFormat();
        DecimalFormatSymbols ds = new DecimalFormatSymbols();
        ds.setDecimalSeparator('.');
        format.setDecimalFormatSymbols(ds);
        format.setMaximumFractionDigits(2);
        format.setMinimumFractionDigits(2);
        return format.format(amount);
    }
}