package org.mmc.utils;

import java.math.BigDecimal;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;

/**
 * Created by: Stoysky
 * Date: 2007-1-4
 * Time: 17:25:58
 *
 * @author <a href="mailto:svenkov@vereo.bg">Stoyo Venkov</a>
 */
public class ParseUtils {
    protected static final String CHAR_YES = "Y";
    protected static final String CHAR_NO = "N";
    private static final String EMPTY_STRING = "";

    public static boolean isEmptyString(String str) {
        return (str == null || str.length() == 0);
    }

    public static Date parseDate(String dateStr) {
        try {
            return (isEmptyString(dateStr) ?
                    null :
                    DateTimeUtils.getDate(dateStr));
        } catch (ParseException e) {

        }
        return null;
    }

    public static Date parseDate(String dateStr, String formatter) {
        try {
            return (isEmptyString(dateStr) ?
                    null :
                    DateTimeUtils.getDate(dateStr, formatter));
        } catch (ParseException e) {

        }
        return null;
    }

    public static BigDecimal parseBigDecimal(String number) {
        return (isEmptyString(number) ? null : NumberUtils.parseBigDecimalComma(number));
    }

    public static Long parseLong(String number) {
        return (isEmptyString(number) ? null : new Long(number));
    }

    public static Integer parseInteger(String number) {
        return (isEmptyString(number) ? null : new Integer(number));
    }

    public static String parseString(String str) {
        return (isEmptyString(str) ? null : str);
    }


    public static String parseString(BigDecimal number) {
        return ((number == null) ? EMPTY_STRING : number.toString());
    }

    public static String parseString(Integer number) {
        return ((number == null) ? EMPTY_STRING : number.toString());
    }

    public static String parseString(Long number) {
        return ((number == null) ? EMPTY_STRING : number.toString());
    }

    public static String parseDouble(Double number) {
        if (number == null) {

            return null;

        } else {

            DecimalFormat df = new DecimalFormat("#0.00");
            DecimalFormatSymbols ds = new DecimalFormatSymbols();
            ds.setDecimalSeparator('.');
            df.setDecimalFormatSymbols(ds);
            return df.format(number);

        }
    }

    public static String parseBooleanYN(Boolean value) {
        if(value == null) {
            return null;
        }
        if(value.booleanValue()) {
            return "Y";
        }
        return "N";
    }

    public static String parseToBgYN(Boolean value) {
        if(value == null) {
            return null;
        } else {
            return (value.booleanValue() ? "" : "");
        }

    }

    public static String convertBigDecimal(BigDecimal number) {
        return (number == null ? null : NumberUtils.formatBigDecimalComma(number));
    }

    public static String convertLong(Long number) {
        return (number == null ? null : number.toString());
    }

    public static String convertDate(Date date) {
        return (date == null ? null : DateTimeUtils.formatDate(date));
    }

    public static String convertString(String str) {
        return (isEmptyString(str) ? null : str);
    }

    public static String convertBigDecimalEmpty(BigDecimal number) {
        return (number == null ?
                EMPTY_STRING : NumberUtils.formatBigDecimalComma(number));
    }

    public static String convertBigNumberEmpty(BigDecimal number) {
        return (number == null ? EMPTY_STRING : number.toString());
    }

    public static String convertLongEmpty(Long number) {
        return (number == null ? EMPTY_STRING : number.toString());
    }

    public static String convertDateEmpty(Date date) {
        return (date == null ? EMPTY_STRING : DateTimeUtils.formatDate(date));
    }

    public static String convertStringEmpty(String str) {
        return (isEmptyString(str) ? EMPTY_STRING : str);
    }

    /**
     * Converts a boolean expression to a single character string
     *
     * @param condition boolean expression
     * Case: <code>condition == true</code> - 'Y'
     * <br>
     * Case: otherwise - 'N'
     */
    public static String convertToYNChar(boolean condition) {
        return (condition ? CHAR_YES : CHAR_NO);
    }

    /**
     * Converts a boolean expression to a single character string
     *
     * @param condition boolean expression
     * Case: <code>condition == true</code> - 'Y'
     * <br>
     * Case: otherwise - 'N'
     */
    public static String convertToYNChar(Boolean condition) {
        if(condition == null) {
            return "";
        } else {
            return (condition.booleanValue() ? CHAR_YES : CHAR_NO);
        }
    }

    /**
     * Converts a single character string to a boolean expression.
     *
     * @param yesNoString a single character string with two possible values -
     * <code>'Y'</code> or <code>'N'</code>
     *
     * @return Case: <code>yesNoString == 'Y'</code> - true
     *         <br>
     *         Case: otherwise - false
     */
    public static boolean convertToPrimBoolean(String yesNoString) {
        return (CHAR_YES.equalsIgnoreCase(yesNoString) ? true : false);
    }

    /**
     * Converts a single character string to a Boolean expression.
     *
     * @param yesNoString a single character string with two possible values -
     * <code>'Y'</code> or <code>'N'</code>
     *
     * @return Case: <code>yesNoString == 'Y'</code> - true
     *         <br>
     *         Case: <code>yesNoString is null</code> - null
     *         <br>
     *         Case: otherwise - false
     */
    public static Boolean convertToBoolean(String yesNoString) {
        if(isEmptyString(yesNoString)) {
            return null;
        }
        return (CHAR_YES.equalsIgnoreCase(yesNoString) ? Boolean.TRUE : Boolean.FALSE);
    }

    /**
     * Converts a single character string to a Boolean expression.
     *
     * @param yesNoString a single character string with two possible values -
     * <code>'Y'</code> or <code>'N'</code>
     *
     * @return Case: <code>yesNoString == 'Y'</code> - true
     *         <br>
     *         Case: <code>yesNoString == 'N'</code> - false
     */
    public static Boolean convertToBooleanNoNull(String yesNoString) {
        return (CHAR_YES.equalsIgnoreCase(yesNoString) ? Boolean.TRUE : Boolean.FALSE);
    }

    public static Double convertDouble(String number) {
        return (isEmptyString(number) ? null : new Double(number));
    }

    // Returns the contents of the file in a byte array.
    public static byte[] getBytesFromFile(InputStream is) throws IOException {

        // Get the size of the file
        is.reset();
        long length = is.available();

        // Create the byte array to hold the data
        byte[] bytes = new byte[(int)length];

        // Read in the bytes
        int offset = 0;
        int numRead = 0;
        while (offset < bytes.length
               && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
            offset += numRead;
        }

        // Ensure all the bytes have been read in
        if (offset < bytes.length) {
            throw new IOException("Could not completely read file ");
        }

        // Close the input stream and return bytes
        is.close();
        return bytes;
    }
}