package org.mmc.web.actions.administration;

import org.mmc.web.actions.BaseSecurityAction;
import org.mmc.web.forms.administration.EditUserForm;
import org.mmc.model.domain.Role;
import org.mmc.model.service.UsersService;

import java.util.List;
import java.util.ArrayList;

/**
 * Created on: 2007-9-18
 *
 * @author Stoyo Venkov
 */
public abstract class BaseUserAction extends BaseSecurityAction {
    protected UsersService service;

    public BaseUserAction(UsersService service) {
        this.service = service;
    }

    protected void setUpRoles(EditUserForm form){
        List allRoles = service.getAllRoles();
        String[] userRoleIds = form.getUserRoles();
        List userRolesList = getUserRoles(userRoleIds, allRoles);
        form.setUserRolesList(userRolesList);
        List availableRolesList = getAvailableRoles(userRoleIds, allRoles);
        form.setAvailableRolesList(availableRolesList);
    }

    protected void setUpPoisitions(EditUserForm form) {
        List positions = service.getPositionsService().getAllActive();
        form.setPositionsList(positions);
    }

    /**
     * Filters <code>List</code> of {@link org.mmc.model.domain.Role} by <code>roleId</code> against an array of roleIds.
     *
     * @param userRoleIds filters, contains role ids in text forms
     * @param allRoles - all roles that exist in the system,
     * <code>List</code> of <codeRole.class</code> instances
     *
     * @return <code>List</code> of <code>Role</code> where each role in the list
     *         has a matching id in the filters
     *
     */
    protected List getUserRoles(String[] userRoleIds, List allRoles) {
        List filteredRoles = new ArrayList();
        if (userRoleIds == null) {
            return filteredRoles;
        }

        Role role = null;
        Long roleId;
        String strRoleId;
        String roleIdFilter;

        for (int allRolesIndex = 0; allRolesIndex < allRoles.size(); allRolesIndex++) {
            role = (Role) allRoles.get(allRolesIndex);

            for (int filterIndex = 0; filterIndex < userRoleIds.length; filterIndex++) {
                roleId = role.getId();
                strRoleId = roleId.toString();
                roleIdFilter = userRoleIds[filterIndex];

                if (roleIdFilter.equals(strRoleId)) {
                    filteredRoles.add(role);
                }
            }
        }

        return filteredRoles;

    }

    /**
     * Filters <code>List</code> of {@link Role} by <code>roleId</code> against an array of roleIds.
     *
     * @param userRoleIds filters, contains role ids in text forms
     * @param allRoles - all roles that exist in the system
     *
     * @return <code>List</code> of <code>Role</code> where none of the roles in the list
     *         has a matching id in the filters
     *
     */
    protected List getAvailableRoles(String[] userRoleIds, List allRoles){
        if (userRoleIds == null) {
            return new ArrayList(allRoles);
        }

        List result = new ArrayList();

        for(int i = 0; i < allRoles.size(); i++) {
            Role role = (Role) allRoles.get(i);
            int j;
            for(j = 0; j < userRoleIds.length; j++) {
                String userRoleId = userRoleIds[j];
                if(role.getId().equals(new Long(userRoleId))) {
                    break;
                }
            }
            if(j == userRoleIds.length) {
                result.add(role);
            }
        }

        return result;
    }
}
