package org.mmc.web.actions.administration;

import org.mmc.web.Constants;
import org.mmc.web.forms.administration.EditUserForm;
import org.mmc.model.service.UsersService;
import org.mmc.model.domain.User;
import org.mmc.model.domain.MMCPosition;
import org.mmc.utils.ParseUtils;
import org.mmc.utils.StringUtils;
import org.mmc.utils.PasswordCrypt;
import org.apache.struts.action.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Created on: 2007-9-18
 *
 * @author Stoyo Venkov
 */
public class SaveUserAction extends BaseUserAction {
    public SaveUserAction(UsersService service) {
        super(service);
    }

    protected ActionForward doExecute(ActionMapping actionMapping,
                                      ActionForm actionForm,
                                      HttpServletRequest httpServletRequest,
                                      HttpServletResponse httpServletResponse)
            throws Exception {

        EditUserForm form = (EditUserForm) actionForm;

        ActionErrors errors = form.validate(actionMapping, httpServletRequest);
        if(!errors.isEmpty()) {
            saveErrors(httpServletRequest, (ActionMessages) errors);
        } else {
            User user = createDomainObject(form);
            user = service.saveOrUpdateUser(user);
            form.initializFormFromDomainObject(user);

            ActionMessages messages = new ActionMessages();
            messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("message.userSaved"));
            saveMessages(httpServletRequest, messages);
        }
        setUpRoles(form);
        setUpPoisitions(form);
        return actionMapping.findForward(Constants.FORWARD_SUCCESS);
    }

    private User createDomainObject(EditUserForm form) {
        User result = new User();
        result.setId(ParseUtils.parseLong(form.getId()));
        result.setFirstName(ParseUtils.parseString(form.getFirstName()));
        result.setSecondName(ParseUtils.parseString(form.getSecondName()));
        result.setFamilyName(ParseUtils.parseString(form.getFamilyName()));
        result.setEmail(ParseUtils.parseString(form.getEmail()));
        result.setPhoneNumber(ParseUtils.parseString(form.getPhoneNumber()));
        result.setUserName(ParseUtils.parseString(form.getUserName()));
        if(!StringUtils.isEmpty(form.getPositionId())) {
            Long positionId = ParseUtils.parseLong((form.getPositionId()));
            MMCPosition position = service.getPositionsService().getPositionById(positionId.longValue());
            result.setPosition(position);
        }

        result.setPassword(PasswordCrypt.digest(form.getPassword1()));

        result.setActive(ParseUtils.convertToBooleanNoNull(form.getActive()));
        result.setVersion(ParseUtils.parseLong(form.getVersion()));

        result.setRoles(getUserRoles(form.getUserRoles(), service.getAllRoles()));

        return result;
    }
}
