package org.mmc.web.actions.probes;

import org.apache.struts.action.*;
import org.mmc.model.service.NomenclaturesService;
import org.mmc.model.service.ProbesService;
import org.mmc.model.domain.Probe;
import org.mmc.model.domain.User;
import org.mmc.web.forms.probes.RegisterProbeForm;
import org.mmc.web.Constants;
import org.mmc.utils.ParseUtils;
import org.mmc.utils.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/**
 * Created on: 2007-10-10
 *
 * @author Stoyo Venkov
 */
public class RegisterProbeAction extends BaseProbeAction {
    private ProbesService probesService;

    public RegisterProbeAction(ProbesService probesService, NomenclaturesService nomenclaturesService) {
        super(nomenclaturesService);
        this.probesService = probesService;
    }

    protected ActionForward doExecute(ActionMapping actionMapping,
                                      ActionForm actionForm,
                                      HttpServletRequest httpServletRequest,
                                      HttpServletResponse httpServletResponse)
            throws Exception {

        RegisterProbeForm form = (RegisterProbeForm) actionForm;

        ActionErrors errors = form.validate(actionMapping, httpServletRequest);
        if(!errors.isEmpty()) {
            saveErrors(httpServletRequest, (ActionMessages) errors);
        } else {
            Long patientId = ParseUtils.parseLong(form.getPatientId());
            Long requestId = ParseUtils.parseLong(form.getRequestId());
            List criteria = getChosenList(form.getChosenCriteria(),
                    nomenclaturesService.getCriteriaService().getAllCriteria());
            List projects = getChosenList(form.getChosenProjects(),
                    nomenclaturesService.getProjectsService().getAllActive());
            Integer termInDays = ParseUtils.parseInteger(form.getAnalysisTerm());
            Long probeTypeId = ParseUtils.parseLong(form.getProbeTypeId());
            Long tubeId = ParseUtils.parseLong(form.getTubeId());
            User user = getLoggedUser(httpServletRequest);
            Probe probe = probesService.registerProbe(user.getId(), patientId, requestId, criteria, projects, termInDays, probeTypeId, tubeId);

            ActionMessages messages = new ActionMessages();
            messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("message.probeRegistered", probe.getId()));
            saveMessages(httpServletRequest, messages);
        }

        loadProbeTypes(form);
        loadCriteria(form);
        loadProjects(form);

        return actionMapping.findForward(Constants.FORWARD_SUCCESS);
    }
}
