package org.mmc.web.filters;

import org.mmc.model.domain.User;
import org.mmc.model.service.UsersService;
import org.mmc.web.Constants;
import org.mmc.utils.StringUtils;
import org.mmc.utils.PasswordCrypt;

import javax.servlet.*;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 * Created by: Stoyo Venkov
 * <br/>
 * Created on: 2007-9-8
 */
public class SecurityFilter implements Filter {
    private static final String PAGE_LOGIN = "/login.do";

    private static final String ACTION_NAME = "action";
    private static final String ACTION_LOGIN_VALUE = "login";
    private static final String USERNAME_NAME = "loginUsername";
    private static final String PASSWORD_NAME = "loginPassword";
    private static final String ERRORS_NAME = "errors";

    private FilterConfig filterConfig;

    private UsersService service;

    public UsersService getService() {
        return service;
    }

    public void setService(UsersService service) {
        this.service = service;
    }

    public void init(FilterConfig filterConfig) throws ServletException {
        this.filterConfig = filterConfig;
    }

    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {

        HttpServletRequest req = (HttpServletRequest) servletRequest;
        HttpSession session = req.getSession();

        User loggedUser = (User) session.getAttribute(Constants.LOGGED_USER);

        if (loggedUser == null) {
            if ((servletRequest.getParameter(ACTION_NAME) != null) &&
                (servletRequest.getParameter(ACTION_NAME).equals(ACTION_LOGIN_VALUE))) {
                String username = req.getParameter(USERNAME_NAME);
                String password = req.getParameter(PASSWORD_NAME);

                if (StringUtils.isEmpty(username)) {
                    servletRequest.setAttribute(ERRORS_NAME, "error.noUserName");
                    servletRequest.getRequestDispatcher(PAGE_LOGIN).forward(servletRequest, servletResponse);
                    return;
                }

                if (StringUtils.isEmpty(password)) {
                    servletRequest.setAttribute(ERRORS_NAME, "error.noPassword");
                    servletRequest.getRequestDispatcher(PAGE_LOGIN).forward(servletRequest, servletResponse);
                    return;
                }

                loggedUser = service.getUserByUsernameAndPass(PasswordCrypt.digest(password), username);

                if(loggedUser == null) {
                    servletRequest.setAttribute(ERRORS_NAME, "error.wrongPassOrUsername");
                    servletRequest.getRequestDispatcher(PAGE_LOGIN).forward(servletRequest, servletResponse);
                    return;
                }

                if(!loggedUser.getActive().booleanValue()) {
                    servletRequest.setAttribute(ERRORS_NAME, "error.userInactive");
                    servletRequest.getRequestDispatcher(PAGE_LOGIN).forward(servletRequest, servletResponse);
                    return;
                }

                session.setAttribute(Constants.LOGGED_USER, loggedUser);
            } else {
                servletRequest.getRequestDispatcher(PAGE_LOGIN).forward(servletRequest, servletResponse);
                return;
            }
        }

        filterChain.doFilter(servletRequest, servletResponse);
    }

    public void destroy() {
        this.filterConfig = null;
    }
}

