package org.mmc.web.forms;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMapping;
import org.mmc.utils.StringUtils;
import org.mmc.utils.DateTimeUtils;
import org.mmc.utils.NumberUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.ResourceBundle;
import java.text.ParseException;

/**
 * Created on: 2007-9-9
 *
 * @author Stoyo Venkov
 */
public abstract class BaseValidatorForm extends ActionForm {
    private final ResourceBundle resourceFile = ResourceBundle.getBundle("application");

    protected void addError(ActionErrors errors, String errorKey) {
        ActionMessage message = new ActionMessage(errorKey);
        errors.add(ActionErrors.GLOBAL_MESSAGE, message);
    }

    protected void addError(ActionErrors errors, String errorKey, String fieldKey) {
        ActionMessage message = new ActionMessage(errorKey, getMsgFromResources(fieldKey));
        errors.add(ActionErrors.GLOBAL_MESSAGE, message);
    }

    protected void addParamError(ActionErrors errors, String errorKey, String fieldName) {
        ActionMessage message = new ActionMessage(errorKey, fieldName);
        errors.add(ActionErrors.GLOBAL_MESSAGE, message);
    }

    protected void checkRequiredStrField(ActionErrors errors, String fieldValue, String fieldKey) {
        if(StringUtils.isEmpty(fieldValue)) {
            addError(errors, "error.obligatoryField", fieldKey);
        }
    }

    protected void checkRequiredParamValue(ActionErrors errors, String fieldValue, String paramName) {
        if(StringUtils.isEmpty(fieldValue)) {
            addParamError(errors, "error.obligatoryParamValue", paramName);
        }
    }

    protected void checkDateField(ActionErrors errors, String fieldValue, String fieldKey) {
        if(!StringUtils.isEmpty(fieldValue)) {
            try {
                DateTimeUtils.getDate(fieldValue);
            } catch (ParseException e) {
                addError(errors, "error.invalidDateMask", fieldKey);
            }
        }
    }

    protected void checkLongField(ActionErrors errors, String fieldValue, String fieldKey) {
        if(!StringUtils.isEmpty(fieldValue)) {
            try {
                NumberUtils.parseLong(fieldValue);
            } catch (NumberFormatException e) {
                addError(errors, "error.numberField", fieldKey);
            }
        }
    }

    protected String getMsgFromResources(String aKey) {
        return resourceFile.getString(aKey);
    }
}
